"""
Created on Tue Sep  8 15:47:03 2020

@author: rdvan
Function for parameter sensitivity analysis 
"""

import numpy as np
import scipy.integrate as spi
import matplotlib.pyplot as plt
from model1_parameters import *
from model1_gut import system_toxin

def perform_psa(system, p, r, t, x0, solve_method, xs0_equals_C=False):
    """
    Function that performs a paramter sensitivity analysis based on a given 
    system of ODEs, paramters, range of sensitivity, integration boundaries,
    initial conditions and solving method.

    Parameters
    ----------
    system : TYPE function
        DESCRIPTION. should be a function defining the system of ODEs witn an 
        input being the parameters in a list/array format
    p : TYPE np.array 
        DESCRIPTION. should be list of paramters that have to be changed in the
        analysis
    r : TYPE list or tuple with len = 2
        DESCRIPTION. indicates range of parameter sensitivity analysis. For 
        example, r = 0.5,1.5 indicates a range of 50 till 150 %
    t : TYPE list or tuple with len = 3
        DESCRIPTION. integration boundaries with the amount of steps where
        the ode system has to be evaluated
    x0 : TYPE list or tuple
        DESCRIPTION. values for the initial condtions of the ode system
    solve_method : TYPE string
        DESCRIPTION. solve_ivp method

    Returns
    -------
    Max_T_all : TYPE np.array
        DESCRIPTION. Holds maximum toxin concentration values for each 
        parameter over the varied interval
    Integ_T_all : TYPE np.array
        DESCRIPTION. Holds accumulated toxin concentration values for each 
        parameter over the varied interval
    range_array : TYPE np.array
        DESCRIPTION. range over which the parameters were varied returned for
        plotting

    """
     
    range_array = np.arange(r[0], r[1], r[2])
    tspan = np.linspace(t[0],t[1],num=t[2])
    
    Max_T_all = np.empty((0,len(range_array)))
    Integ_T_all = np.empty((0,len(range_array)))
    Max_RNA_all = np.empty((0,len(range_array)))
    Max_t_all_cry = np.empty((0,len(range_array)))
    Max_t_all_RNA = np.empty((0,len(range_array)))
    
    
    
    for i in range(len(p)):
        max_T_value = np.empty(0)
        int_T_value = np.empty(0)
        max_RNA_value = np.empty(0)
        time_to_max_cry = np.empty(0)
        time_to_max_RNA = np.empty(0)
        for j in range(len(range_array)):
            temp_params = np.copy(p)
            temp_params[i] = temp_params[i]*range_array[j]
            
            if xs0_equals_C:
                x0[0] = temp_params[1]
            
            sol = spi.solve_ivp(lambda t, y: system(t,y,temp_params), [t[0],t[1]], x0, method='RK45', t_eval=tspan)
            #Extracting the maximum toxin value
            max_T_value = np.append(max_T_value, max(sol.y[3]))
            max_RNA_value = np.append(max_RNA_value, max(sol.y[4]))
            
            #Extracting integeral data - this shows the average toxin value over time
            int_T_value = np.append(int_T_value, np.trapz(sol.y[3], x = sol.t))
            time_to_max_cry = np.append(time_to_max_cry, sol.t[np.argmax(sol.y[3])])
            time_to_max_RNA = np.append(time_to_max_RNA, sol.t[np.argmax(sol.y[4])])
            
            
            

        
        Max_T_all = np.append(Max_T_all, [max_T_value],axis=0)
        Max_RNA_all = np.append(Max_RNA_all,[max_RNA_value],axis =0)
        Integ_T_all = np.append(Integ_T_all, [int_T_value], axis=0)
        Max_t_all_cry = np.append(Max_t_all_cry, [time_to_max_cry], axis=0)
        Max_t_all_RNA = np.append(Max_t_all_RNA, [time_to_max_RNA], axis=0)
        print(len(p) - i)
        
    # change for X_s0
    max_T_value = np.empty(0)
    int_T_value = np.empty(0)
    max_RNA_value = np.empty(0)
    time_to_max_cry = np.empty(0)
    time_to_max_RNA = np.empty(0)
    for k in range(len(range_array)):
        X0_temp = np.copy(x0)
        X0_temp[0] = X0_temp[0]*range_array[k]
        
        sol = spi.solve_ivp(lambda t, y: system(t,y,p), [t[0],t[1]], X0_temp, method='RK45', t_eval=tspan)
        
        #Extracting the maximum toxin value
        max_T_value = np.append(max_T_value, max(sol.y[3]))
        max_RNA_value = np.append(max_RNA_value, max(sol.y[4]))
        #Extracting integeral data - this shows the average toxin value over time
        int_T_value = np.append(int_T_value, np.trapz(sol.y[3], x = sol.t))
        time_to_max_cry = np.append(time_to_max_cry, sol.t[np.argmax(sol.y[3])])
        time_to_max_RNA = np.append(time_to_max_RNA, sol.t[np.argmax(sol.y[4])])

    
            
    Max_T_all = np.append(Max_T_all, [max_T_value],axis=0)
    Max_RNA_all = np.append(Max_RNA_all,[max_RNA_value],axis =0)
    Integ_T_all = np.append(Integ_T_all, [int_T_value], axis=0)
    Max_t_all_cry = np.append(Max_t_all_cry, [time_to_max_cry], axis=0)
    Max_t_all_RNA = np.append(Max_t_all_RNA, [time_to_max_RNA], axis=0)
    
        
        
    return Max_T_all, Integ_T_all, range_array,Max_RNA_all, Max_t_all_cry, Max_t_all_RNA


